# GoodID Connect

A helper JavaScript library to easily implement and use the GoodID service.


## Installation

There are several options to download, install and start using GoodID Connect.

### Install with Bower

GoodID Connect is a registered package on Bower. Install it with the following command:
```
$ bower install goodid
```

The `-S` parameter adds the library to the `bower.json` dependencies section.

This will install it under the bower components directory, `bower_components` is the default name of this dir.

### Install with npm

GoodID Connect is available from the [npm registry](https://www.npmjs.com/package/goodid) as well. You can install it with the command:
```
$ npm install goodid
```

If you add the `--save` switch, it will save it under your `dependencies` section of your `package.json`.

This will add the package `goodid` under your `node_modules` directory. The main file for this is the uncompressed version,
however there is also a minified version with the name `connect.min`.

### Include from CDN

GoodID Connect is available from the following locations:
```
<script src="//cdn.jsdelivr.net/goodid/latest/connect.min.js"></script>
```
CDN is kindly provided by [jsDelivr](https://www.jsdelivr.com/). Check out their [version aliasing doc](https://github.com/jsdelivr/jsdelivr#version-aliasing)
for more options on how to include the library.

The library is also available directly from goodid.net:
```
<script src="http://connect.goodid.net/dist/connect.min.js"></script>
```
For the time being this method only supports HTTP. HTTPS version is coming soon. Note, that at the moment, this is NOT a CDN.

### Clone git repo

If none of the above solutions work for you, or you just want to place this somewhere custom, you can clone this repo with the command:
```
$ git clone https://github.com/idandtrust/goodid.git
```
This way you can just `git pull` to update the lib.

Alternatively, if you want go the extra-manual route, you can just download the lib as a zipped archive
from GitHub [here](https://github.com/idandtrust/goodid/archive/master.zip) and place it whereever you like.


## Usage

GoodID Connect is fully AMD and CommonJS-compatible. Alternatively, you can use its public API in the global JS scope.

### AMD

You can use GoodID Connect with any AMD loader. The following example uses [RequireJS](http://requirejs.org/):

**index.html:**
```html
<script data-main="main" src="path/to/require.js"></script>
```

**main.js:**
```javascript
requirejs.config({
    paths: {
        goodid: 'path/to/goodid/connect' // or connect.min for the minified version
    }
});
requirejs(['app']);
```

Here you need to replace `path/to/goodid/connect` with the actual location, be it a local or CDN-served file.

**app.js:**
```javascript
define(function(require) {
    var goodid = require('goodid');

    goodid.init({
        client_id: '[your client_id here]',
        onlogin: onLogin
    });

    function onLogin(e) {
        if (goodid.isLoginSuccessful()) { /*...*/ }
    }

    return goodid;
});
```

For a complete reference of the RequireJS API and config parameters, visit [this page](http://requirejs.org/docs/api.html).

### CommonJS

You can also use the library with a CommonJS wrapper. Here is an example for that:

```javascript
var goodid = require('./path/to/goodid/connect'); // or connect.min for the minified version

module.exports = function() {
  goodid.init({
    client_id: '[your client_id here]',
    /*...*/
  });
  /*...*/
}
```

Replace `path/to/goodid/connect` with your preferred module file location. See the installation guide for GoodID Connect above.

You can use [Browserify](http://browserify.org/), [Webpack](https://webpack.github.io/) or some other bundler to use these modules in your browser.

### Global object

If you use neither AMD, nor CommonJS, then GoodID Connect exposes a global variable on the `window` object, called `GoodID`.
Insert the following code anywhere within your HTML body:

```html
<script>
'use strict';
function initGoodID() {
    GoodID.init({
        client_id: '[your client_id here]',
        onlogin: onLogin
    });
}

function onLogin(e) { /*...*/ }

(function(d, t, i) {
    var s, b;
    if (d.getElementById(i)) {return;}
    s = d.createElement(t); s.id = i;
    s.src = 'path/to/goodid/connect.min.js';
    s.onload = s.onreadystatechange = initGoodID;
    b = d.getElementsByTagName(t)[0]; b.parentNode.insertBefore(s, b);
})(document, 'script', 'goodid-sdk');
</script>
```

You will need to replace `path/to/goodid/connect.min.js` with your preferred location. For this setup option, we suggest using the
one provided by our CDN-hosted version that's available on this location:
`//cdn.jsdelivr.net/goodid/latest/connect.min.js`. See more installation options in the section above.

## GoodID login button

GoodID works by opening a popup window that shows your users the GoodID authentication screen. To open this popup window,
you can manually call the `signIn()` method of the API (see below).
Another way would be to use the `<goodid:login>` custom HTML tag. A basic example goes like this:
```
<goodid:login onlogin="goodidLogin"></goodid:login>
```
When GoodID Connect loads up, the `<goodid:login>` tag will be replaced by the actual button. After the login status changes,
(be it a successful or a denied login) the `window.goodidLogin` method will be called.

### Specific attributes

The following attributes have special meanings when applied to the `<goodid:login>` tag:

#### onlogin

This parameter specifies a function name on the `window` object that will be run on a successful or a denied login.

#### scope

You can specify which scopes you request from the logged in users in the general configuration. You can also specify on
a per-button basis by entering the scopes as the value of this parameter.

### Other attributes

Any other attributes defined on the `<goodid:login>` tag will be applied to the generated button as is.
One exception to this is the `class` attribute. All buttons will be rendered with the `class="goodid-login"` attribute.
You can define custom classes, however those will be added next to the default `goodid-login` class.


## Public API

The GoodID Connect API exposes the following methods:

### init(config, script)

*returns:* string

*parameters:*
* **config**
   An object that lets you configure your GoodID implementation. See the possible configuration properties below.
* **script**
   An optional parameter that you can use when implementing the global variable-version. In this case, this parameter accepts the `<script>` object that requires the GoodID Connect lib so that it can unload the `onload` and `onreadystatechange` events from the `<script>` object. There's a memory leak when handling these methods in IE prior to 9.

This method initializes GoodID Connect.

### signIn()

*returns:* false

This is an event handler method that you call when you want to open the GoodID popup. For example you can assign this method to a `click` event of a button, then
the GoodID popup window will open when clicking that button.

This method returns `false`, so it will prevent the default event (and stop event propagation, if applied by jQuery) on the target element.

### getAccessToken()

*returns:* string

When an access token is requested to access the userinfo endpoint, you can get its value after a successful sign-in by calling this method.
Returns `null` when no access token can be returned.

### getIdToken()

*returns:* object

After a successful sign-in, the ID token is returned in the URL's fragment. This method will return this token in the following structure:
```
{
  "original": "[the raw value of the ID token]",
  "header": { "alg": "RS256", "typ": "JWT"},
  "payload": {"sub": ...},
  "signature": "[the ID token's signature]"
}
```

The possible contents of the `payload` object can be found in the [OpenID Connect specification](http://openid.net/specs/openid-connect-core-1_0.html#IDToken)

### getClaims(callback)

*returns:* object

*parameters:*

* **callback**
A callback method that runs after the XHR from the userinfo endpoint. Signature: `callback(error, claims)`

On a successful sign-in, this method will, by default return the claims requested in the ID token.
When an access token is requested and the `callback` parameter is supplied, the `getClaimsFromUserinfo(callback)` method will be invoked as well.

### getClaimsFromUserinfo(callback)

*returns:* null

*parameters:*
* **callback**
A function that runs after the XHR from the userinfo endpoint. Signature: `callback(error, claims)`

When an access token is requested, an XHR request will be fired on the userinfo to get these claims.
When this XHR returns, the callback method will be called. The method signature is: `callback(error, claims)`

### getErrors()

*returns:* object

On a failed or a denied sign-in, this method will return the error and description, like this:
```
{
  "error": "[error code]",
  "error_description": "[textual representation of the error]",
  "state": "[state parameter as supplied on the authentication endpoint]"
}
```

### getLoginStatus()

*returns:* string

This method returns a string representing the current login status. You can use this for example in the method called by the `onlogin` configuration parameter.

Return values:
* **success** When the sign-in was successful
* **denied** When the sign-in was denied or failed
* **nologin** When no previous login attempt was found in the current session

### isLoginSuccessful()

*returns:* bool

Returns `true` when the login was successful, `false` otherwise.

## Configuration parameters

Since GoodID is an OpenID Connect-compliant service, most of these parameters refer to parameters sent to the authorization endpoint
of the provider. For further explanation and use-cases, please refer to the [OIDC specs](http://openid.net/specs/openid-connect-core-1_0.html#AuthorizationEndpoint).

### Parameters

#### client_id

**Required**

*type:* string

Your client ID, which you receive when you become a client of GoodID. You can sign up to GoodID on [this page](https://goodid.net/client-reg).

#### redirect_uri

*type:* string

This is the URI on your site where the popup will redirect to with the ID token (on success) or the error (on failure) in the fragment part of the URL.
By default, this will be the same page where the user clicks the GoodID sign-in button.

#### scope

*type:* array or string

The claim scopes you want. Possible scopes are: `profile`, `email`, `address`, `phone` and `openid`.
The expected format is a space-separated string of one or more of these scopes.

#### claims

*type:* object

Specify an object as the value of this parameter.

#### access_token

*type:* boolean

Enter `true`, if you want to request an access token, `false`, if you don't.
If you want to request claims in the userinfo, then you need to have an access token, otherwise the popup will throw an error.

#### state

*type:* string

The state string is an opaque value that is used to maintain state between the request and the callback. This is usually done to mitigate XSRF attacks.
Its value will be returned verbatim within the redirect URL's fragment.

#### nonce

*type:* string

This parameter can be used to send a random string that will be returned unmodified in the ID token's `nonce` claim. This can be used to mitigate replay attacks.

#### onlogin

*type:* function

A callback function that will be invoked when a sign-in event (successful or otherwise) happens. Receives the event object as parameter.

### Default config

```javascript
{
  "client_id": null,
  "redirect_uri": "[current domain + pathname]",
  "scope": "openid",
  "claims": null,
  "access_token": true,
  "state": null,
  "nonce": null,
  "onlogin": null
}
```


## Scopes and claims


## Acknowledgments

* This library uses code from toddmotto's atomic.js library ([GitHub](https://github.com/toddmotto/atomic/blob/master/src/atomic.js), [License](https://github.com/toddmotto/atomic/blob/master/LICENSE))
* Uses the `Object.assign` polyfill from MDN ([Page](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/assign#Polyfill))
* Uses the `CustomEvent` polyfill from MDN ([Page](https://developer.mozilla.org/en-US/docs/Web/API/CustomEvent/CustomEvent#Polyfill))
